from math import sqrt


def q(s=None, p=None, pf=None):
    """Calculates the reactive power `p` (unitless) from the apparent power `s`,
    the active power `p`, or the power factor `pf` (2 values required).
    """

    if pf is not None:
        if not -1 <= pf <= 1:
            raise ValueError("PF must be between -1 and 1.")

    if p is not None and pf is not None:
        return sqrt(p**2 * ((1 / pf**2) - 1))

    if s is not None and pf is not None:
        return sqrt(s**2 - (s * pf) ** 2)

    if p is not None and s is not None:
        return sqrt(s**2 - p**2)

    raise ValueError("You must provide at least two arguments.")


def p(s=None, q=None, pf=None):
    """Calculates the active power `p` (unitless) from the apparent power `s`,
    the reactive power `q`, or the power factor `pf` (2 values required).
    """

    if pf is not None:
        if not -1 <= pf <= 1:
            raise ValueError("PF must be between -1 and 1.")

    if q is not None and pf is not None:
        return sqrt(q**2 / ((1 / pf**2) - 1))

    if s is not None and pf is not None:
        return s * pf

    if q is not None and s is not None:
        return sqrt(s**2 - q**2)

    raise ValueError("You must provide at least two arguments.")



# Standard library
from math import pi
from collections.abc import Iterable

# Current module
from energy_tools.complex import EleComplex


def serie(*args):
    """Puts impedances in serie (yep, simple summation).

    Args:
        z: List of impedances (either complex or real or a combination of both).

    Returns:
        Serie impedance.
    """

    # Check if an iterable was supplied instead of multiple arguments
    if len(args) == 1 and isinstance(args[0], Iterable):
        args = args[0]

    return sum(args)


def parallel(*args):
    """Puts impedances in parallel.

    Args:
        z: List of impedances (either complex or real or a combination of both).

    Returns:
        Parallel impedance.
    """

    # Check if an iterable was supplied instead of multiple arguments
    if len(args) == 1 and isinstance(args[0], Iterable):
        args = args[0]

    try:
        return 1 / sum([1.0 / y for y in args])

    except ZeroDivisionError:
        return 0.0


def zCap(C, f=60):
    """Retourne l'impédance complexe d'un banc de condensateurs en fonction de
    la capacitance et de la fréquence (à 60Hz par défaut).
    """
    return EleComplex(1 / (2j * pi * f * C))


def zInd(L, f=60):
    """Retourne l'impédance complexe d'une inductance en fonction de
    l'inductance et de la fréquence (à 60Hz par défaut).
    """
    return EleComplex(2j * pi * f * L)
